from collections import OrderedDict

import mock
import pytest
from django.contrib.staticfiles.storage import staticfiles_storage

from easy_select2 import widgets


def test_select2mixin_constructor():
    s = widgets.Select2Mixin()
    assert isinstance(s.select2attrs, dict)
    assert s.select2attrs['width'] == '250px'

    s1 = widgets.Select2Mixin(select2attrs={'auto': True})
    assert s1.select2attrs['auto'] == True

    with pytest.raises(AssertionError):
        widgets.Select2Mixin(select2attrs="wrong value")


def test_select2mixin_get_options():
    s = widgets.Select2Mixin()
    assert isinstance(s.get_options(), dict)


def test_select2mixin_render_options_code():
    s = widgets.Select2Mixin()
    options = OrderedDict()
    options['number'] = 1
    options['string'] = u'string'
    options['dict'] = {'k': 'v'}
    options['list'] = [1, 'two']
    result = s.render_select2_options_code(options, 'some_id')
    expected = (
        "data-number='1' "
        "data-string='string' "
        "data-dict='{\"k\": \"v\"}' "
        "data-list='[1, \"two\"]'"
    )
    assert result == expected


@mock.patch.object(widgets.Select2Mixin, 'render_select2_options_code')
def test_select2mixin_render_js_code(mocked):
    mocked.return_value = "some_options"
    s = widgets.Select2Mixin()
    s.html = "{id}: {options}"
    result = s.render_js_code("some_id")
    expected = "some_id: some_options"
    assert result == expected


def test_select2mixin_render_js_code_should_return_empty_string():
    s = widgets.Select2Mixin()
    assert s.render_js_code(id_=None) == u''


class SuperWithRender(object):
    def render(self, *args, **kwargs):
        return "super"


class TestRender(widgets.Select2Mixin, SuperWithRender):
    pass


@mock.patch.object(TestRender, 'render_js_code')
def test_select2mixin_render(mocked):
    mocked.return_value = "some_js"
    s = TestRender()
    result = s.render('name', 'value', attrs={'id': 'some_id'})
    expected = "supersome_js"
    assert result == expected


def test_staticfiles_url(settings):
    js = widgets.SELECT2_WIDGET_JS
    css = widgets.SELECT2_WIDGET_CSS

    def all_startswith(string, iterable):
        return all([staticfiles_storage.url(x).startswith(string)
                    for x in iterable])

    assert all_startswith(
            settings.STATIC_URL,
            js,
    )
    for v in css.values():
        assert all_startswith(
                settings.STATIC_URL,
                v,
        )



from __future__ import print_function

import pytest

import mock
from django import forms

from easy_select2 import forms as es2_forms
from easy_select2 import utils
from easy_select2 import widgets
from demoapp import models as m


def test_apply_select2():
    cls = utils.apply_select2(forms.Select)
    assert cls.__name__ == 'Select2Select'
    assert cls.__bases__ == (widgets.Select2Mixin, forms.Select)


def test_select2_modelform_meta_empty_widgets():
    cm = utils.select2_modelform_meta(m.EmptyModel)
    assert cm.__name__ == 'Meta'
    assert cm.__bases__ == (object,)
    assert cm.model == m.EmptyModel
    assert cm.widgets == {}


def test_select2_modelform_meta_widgets_without_arguments():
    cm = utils.select2_modelform_meta(m.TestFieldsModel)
    assert cm.__name__ == 'Meta'
    assert cm.__bases__ == (object,)
    assert cm.model == m.TestFieldsModel
    assert cm.widgets != {}
    assert isinstance(cm.widgets['fk_field'], widgets.Select2)
    assert isinstance(cm.widgets['choice_field'], widgets.Select2)
    assert isinstance(cm.widgets['m2m_field'], widgets.Select2Multiple)
    assert 'text' not in cm.widgets


def test_select2_modelform_meta_with_arguments():
    cm = utils.select2_modelform_meta(
            m.TestFieldsModel,
            meta_fields={'extra_field': 1},
            widgets={
                'text': forms.TextInput,
                'fk_field': forms.TextInput,  # must change to Select2
            },
    )
    assert cm.extra_field == 1
    assert cm.widgets['text'] == forms.TextInput
    assert isinstance(cm.widgets['fk_field'], widgets.Select2)


def test_select2_modelform_meta_kwargs():
    cm = utils.select2_modelform_meta(m.TestFieldsModel)
    assert cm.exclude == []
    cm = utils.select2_modelform_meta(
            m.TestFieldsModel,
            fields=['text'],
    )
    assert cm.fields == ['text']
    cm = utils.select2_modelform_meta(
            m.TestFieldsModel,
            exclude=['text'],
    )
    assert cm.exclude == ['text']


# Select2 has already been imported by easy_select2.utils module
@mock.patch('easy_select2.utils.Select2Multiple')
@mock.patch('easy_select2.utils.Select2')
def test_select2_modelform_meta_with_blank_attrs_argument(
        Select2Mock, Select2MultipleMock):
    utils.select2_modelform_meta(m.TestFieldsModel)
    Select2Mock.assert_called_with(select2attrs=None)
    assert Select2Mock.call_count == 2
    Select2MultipleMock.assert_called_with(select2attrs=None)
    assert Select2MultipleMock.call_count == 1


# Select2 has already been imported by easy_select2.utils module
@mock.patch('easy_select2.utils.Select2Multiple')
@mock.patch('easy_select2.utils.Select2')
def test_select2_modelform_meta_with_attrs_argument(
        Select2Mock, Select2MultipleMock):
    attrs = {'attribute': True}
    utils.select2_modelform_meta(m.TestFieldsModel, attrs=attrs)
    Select2Mock.assert_called_with(select2attrs=attrs)
    assert Select2Mock.call_count == 2
    Select2MultipleMock.assert_called_with(select2attrs=attrs)
    assert Select2MultipleMock.call_count == 1


def test_select2_modelform():
    form_class = utils.select2_modelform(m.TestFieldsModel)
    assert form_class.__name__ == 'TestFieldsModelForm'
    assert issubclass(form_class, forms.ModelForm)
    assert hasattr(form_class, 'Meta') == True


@pytest.mark.django_db
def test_select2_modelform_renders_successfully():
    form_class = utils.select2_modelform(m.TestFieldsModel)
    form = form_class()
    assert 'select2' in form.as_p()


def test_select2_modelform_form_class_argument():
    class TestForm(forms.ModelForm):
        pass

    form_class = utils.select2_modelform(
            m.TestFieldsModel,
            form_class=TestForm,
    )
    assert issubclass(form_class, TestForm)

    form_class = utils.select2_modelform(m.TestFieldsModel)
    assert issubclass(form_class, es2_forms.FixedModelForm)


@mock.patch('easy_select2.utils.select2_modelform_meta')
def test_select2_modelform_attrs_argument(mock):
    utils.select2_modelform(m.TestFieldsModel)
    mock.assert_called_with(m.TestFieldsModel, attrs=None)
    attrs = {'attr': False}
    utils.select2_modelform(m.TestFieldsModel, attrs=attrs)
    mock.assert_called_with(m.TestFieldsModel, attrs=attrs)



from easy_select2.utils import select2_modelform_meta, select2_modelform, \
    apply_select2
from easy_select2.widgets import Select2Mixin, Select2, Select2Multiple

__all__ = ['Select2', 'Select2Mixin', 'Select2Multiple', 'apply_select2',
           'select2_modelform', 'select2_modelform_meta']



# coding: utf-8

from easy_select2.forms import FixedModelForm
from demoapp.models import TestFieldsModel


class TestFixedModelForm(FixedModelForm):
    class Meta:
        model = TestFieldsModel
        exclude = []


def test_fixedmodelform():
    form = TestFixedModelForm()
    assert form.fields['fk_field'].help_text.strip() == ''
    assert form.fields['m2m_field'].help_text.strip() == ''
